#include "ColorCubeWindow.h"
#include <iostream>
#include <windows.h>
#include <math.h>
#include <GL/gl.h>

ColorCubeWindow::ColorCubeWindow(int width, int height, char *title)
    :Fl_Gl_Window(width, height, title)
{
    theta = 0.0;
    phi = 3.142 * 0.5;
    range = 1.0;
}

ColorCubeWindow::ColorCubeWindow(int x, int y, int width, int height,
				 char *title)
    :Fl_Gl_Window(x, y, width, height, title)
{
    theta = 0.0;
    phi = 3.142 * 0.5;
    range = 1.0;
}


void
ColorCubeWindow::Draw_X_Face(float y_min, float y_max, float z_min, float z_max,
                             float x)
{
    int	    ny, nz;
    float   sy, sz;
    float   y, z;
    int     i, j;

    ny = (int)ceil((y_max - y_min) / 0.1);
    nz = (int)ceil((z_max - z_min) / 0.1);
    sy = ( y_max - y_min ) / (float)ny;
    sz = ( z_max - z_min ) / (float)nz;

    glBegin(GL_QUADS);
    for ( i = 0, y = y_min ; i < ny ; i++, y += sy )
    {
	for ( j = 0, z = z_min ; j < nz ; j++, z += sz )
	{
	    glColor3f(x, y, z);
	    glVertex3f(x, y, z);
	    glColor3f(x, y + sy, z);
	    glVertex3f(x, y + sy, z);
	    glColor3f(x, y + sy, z + sz);
	    glVertex3f(x, y + sy, z + sz);
	    glColor3f(x, y, z + sz);
	    glVertex3f(x, y, z + sz);
	}
    }
    glEnd();
}


void
ColorCubeWindow::Draw_Y_Face(float x_min, float x_max, float z_min, float z_max,
                             float y)
{
    int	    nx, nz;
    float   sx, sz;
    float   x, z;
    int     i, j;

    nx = (int)ceil((x_max - x_min) / 0.1);
    nz = (int)ceil((z_max - z_min) / 0.1);
    sx = ( x_max - x_min ) / (float)nx;
    sz = ( z_max - z_min ) / (float)nz;

    glBegin(GL_QUADS);
    for ( i = 0, x = x_min ; i < nx ; i++, x += sx )
    {
	for ( j = 0, z = z_min ; j < nz ; j++, z += sz )
	{
	    glColor3f(x, y, z);
	    glVertex3f(x, y, z);
	    glColor3f(x + sx, y, z);
	    glVertex3f(x + sx, y, z);
	    glColor3f(x + sx, y, z + sz);
	    glVertex3f(x + sx, y, z + sz);
	    glColor3f(x, y, z + sz);
	    glVertex3f(x, y, z + sz);
	}
    }
    glEnd();
}


void
ColorCubeWindow::Draw_Z_Face(float x_min, float x_max, float y_min, float y_max,
                             float z)
{
    int	    nx, ny;
    float   sx, sy;
    float   x, y;
    int     i, j;

    nx = (int)ceil((x_max - x_min) / 0.1);
    ny = (int)ceil((y_max - y_min) / 0.1);
    sx = ( x_max - x_min ) / (float)nx;
    sy = ( y_max - y_min ) / (float)ny;

    glBegin(GL_QUADS);
    for ( i = 0, x = x_min ; i < nx ; i++, x += sx )
    {
	for ( j = 0, y = y_min ; j < ny ; j++, y += sy )
	{
	    glColor3f(x, y, z);
	    glVertex3f(x, y, z);
	    glColor3f(x + sx, y, z);
	    glVertex3f(x + sx, y, z);
	    glColor3f(x + sx, y + sy, z);
	    glVertex3f(x + sx, y + sy, z);
	    glColor3f(x, y + sy, z);
	    glVertex3f(x, y + sy, z);
	}
    }
    glEnd();
}


void
ColorCubeWindow::draw(void)
{
    if ( ! valid() )
    {
	glViewport(0, 0, w(), h());
	glLoadIdentity();
	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);
	glClearColor(0.5, 0.5, 0.5, 1.0);
	glEnable(GL_DEPTH_TEST);
    }

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glPushMatrix();

    glRotated((phi * 180.0 / 3.142) - 90.0, 1.0, 0.0, 0.0);
    glRotated(theta * 180.0 / 3.142, 0.0, 1.0, 0.0);
    glTranslatef(-0.5, -0.5, -0.5);

    Draw_X_Face(0.0, range, 0.0, range, 0.0);
    Draw_X_Face(0.0, range, 0.0, range, range);
    Draw_Y_Face(0.0, range, 0.0, range, 0.0);
    Draw_Y_Face(0.0, range, 0.0, range, range);
    Draw_Z_Face(0.0, range, 0.0, range, 0.0);
    Draw_Z_Face(0.0, range, 0.0, range, range);

    glPopMatrix();

}


void
ColorCubeWindow::Button_Down(void)
{
    start_x = Fl::event_x();
    start_y = Fl::event_y();
    if ( start_y == 0 ) start_y++;
    if ( start_y == h() ) start_y--;
    scale_to_top = (3.142 - phi) / (double)start_y;
    scale_to_bottom = phi / ((double)h() - start_y);
    scale_horiz = 2 * 3.142 / (double)w();
    init_theta = theta;
    init_phi = phi;
}


void
ColorCubeWindow::Button_Drag(void)
{
    int	    x = Fl::event_x();
    int     y = Fl::event_y();

    if ( y >= h() )
        y = h() - 1;
    if ( y <= 0 ) y = 1;

    theta = init_theta - (start_x - x) * scale_horiz;
    phi = init_phi
        - (start_y - y ) * ((y < start_y) ? scale_to_top : scale_to_bottom);

    invalidate();
    Fl::redraw();
}


int
ColorCubeWindow::handle(int e)
{
    switch ( e )
    {
      case FL_PUSH:
	Button_Down();
	return 1;
      case FL_DRAG:
      	Button_Drag();
	return 1;
    }

    return Fl_Window::handle(e);
}


